#!/usr/bin/env python3
"""Generates fixed voltages

Voltages can be configured via an optional JSON file
"""


from europi import *
from europi_script import EuroPiScript

import configuration


class OffsetVoltages(EuroPiScript):
    def __init__(self):
        super().__init__()


    @classmethod
    def config_points(cls):
        """Return the static configuration options for this class
        """
        def restrict_voltage(v):
            """If the max output voltage has been lowered, disable the too-high output
            """
            if v > europi_config.MAX_OUTPUT_VOLTAGE:
                return 0.0
            return v

        return [
            configuration.floatingPoint(
                name="CV1",
                minimum=0.0,
                maximum=europi_config.MAX_OUTPUT_VOLTAGE,
                default=restrict_voltage(0.5)
            ),
            configuration.floatingPoint(
                name="CV2",
                minimum=0.0,
                maximum=europi_config.MAX_OUTPUT_VOLTAGE,
                default=restrict_voltage(1.0)
            ),
            configuration.floatingPoint(
                name="CV3",
                minimum=0.0,
                maximum=europi_config.MAX_OUTPUT_VOLTAGE,
                default=restrict_voltage(2.0)
            ),
            configuration.floatingPoint(
                name="CV4",
                minimum=0.0,
                maximum=europi_config.MAX_OUTPUT_VOLTAGE,
                default=restrict_voltage(2.5)
            ),
            configuration.floatingPoint(
                name="CV5",
                minimum=0.0,
                maximum=europi_config.MAX_OUTPUT_VOLTAGE,
                default=restrict_voltage(5.0)
            ),
            configuration.floatingPoint(
                name="CV6",
                minimum=0.0,
                maximum=europi_config.MAX_OUTPUT_VOLTAGE,
                default=restrict_voltage(10.0)
            ),
        ]

    def main(self):
        oled.fill(0)
        oled.show()

        cv1.voltage(self.config.CV1)
        cv2.voltage(self.config.CV2)
        cv3.voltage(self.config.CV3)
        cv4.voltage(self.config.CV4)
        cv5.voltage(self.config.CV5)
        cv6.voltage(self.config.CV6)

        while True:
            pass



"""Logic gates for the EuroPi

Treats ain as a digital input with a threshold of 0.8V

@author Chris Iverach-Brereton <ve4cib@gmail.com>
@date   2023-02-13
"""

from europi import *
from europi_script import EuroPiScript

from experimental.screensaver import OledWithScreensaver

ssoled = OledWithScreensaver()

## Anything above this threshold is considered ON for the analog
#  input
AIN_VOLTAGE_CUTOFF = 0.8

class Logic(EuroPiScript):
    """The main workhorse of the whole module

    Does all 6 basic logic operations, sending the result to each of the
    6 outputs
    """
    def __init__(self):
        super().__init__()

    @classmethod
    def display_name(cls):
        return "Logic"

    def main(self):
        """The main loop

        Connects event handlers for clock-in and button presses
        and runs the main loop
        """

        @b1.handler
        def on_b1_press():
            ssoled.notify_user_interaction()

        @b2.handler
        def on_b2_press():
            ssoled.notify_user_interaction()

        while True:

            # read both inputs as 0/1
            x = din.value() | b1.value()
            y = (1 if ain.read_voltage() > AIN_VOLTAGE_CUTOFF else 0) | b2.value()

            x_and_y = x & y
            x_or_y = x | y
            x_xor_y = x ^ y
            x_nand_y = abs(x_and_y - 1) # bit of a hack to get the inverted values
            x_nor_y = abs(x_or_y -1)    # ~ results in some -2 results, which we don't want
            x_xnor_y = abs(x_xor_y -1)  # so some simple int math will suffice

            cv1.value(x_and_y)
            cv2.value(x_or_y)
            cv3.value(x_xor_y)
            cv4.value(x_nand_y)
            cv5.value(x_nor_y)
            cv6.value(x_xnor_y)
            display_txt = " &:{0}  |:{1}  ^:{2}\n!&:{3} !|:{4} !^:{5}".format(
                x_and_y,
                x_or_y,
                x_xor_y,
                x_nand_y,
                x_nor_y,
                x_xnor_y
            )

            ssoled.centre_text(display_txt)

if __name__ == "__main__":
    Logic().main()



"""See menu.md for details."""
# Reset the module state and display bootsplash screen.
from europi import bootsplash


bootsplash()


from bootloader import BootloaderMenu
from collections import OrderedDict

## Scripts that are included in the menu
#
#  Keys are the names displayed in the menu, values are the fully-qualified names
#  of the classes to launch.  The classes MUST be EuriPiScript subclasses
#
#  The OLED can display up to 16 characters horizontally, so make sure the names fit
#  that width requirement
#
# fmt: off
EUROPI_SCRIPTS = OrderedDict([
#   ["0123456789abcdef",  "contrib.spam.Eggs"],
    ["Arpeggiator",       "contrib.arp.Arpeggiator"],
    ["Bernoulli Gates",   "contrib.bernoulli_gates.BernoulliGates"],
    ["Bezier Curves",     "contrib.bezier.Bezier"],
    ["Bit Garden",        "contrib.bit_garden.BitGarden"],
    ["Clock Modifier",    "contrib.clock_mod.ClockModifier"],
    ["Coin Toss",         "contrib.coin_toss.CoinToss"],
    ["Consequencer",      "contrib.consequencer.Consequencer"],
    ["Conway",            "contrib.conway.Conway"],
    ["CVecorder",         "contrib.cvecorder.CVecorder"],
    ["Daily Random",      "contrib.daily_random.DailyRandom"],
    ["DCSN-2",            "contrib.dscn2.Dcsn2"],
    ["EgressusMelodiam",  "contrib.egressus_melodiam.EgressusMelodiam"],
    ["EnvelopeGen",       "contrib.envelope_generator.EnvelopeGenerator"],
    ["Euclid",            "contrib.euclid.EuclideanRhythms"],
    ["Gates & Triggers",  "contrib.gates_and_triggers.GatesAndTriggers"],
    ["Gate Phaser",       "contrib.gate_phaser.GatePhaser"],
    ["Hamlet",            "contrib.hamlet.Hamlet"],
    ["HarmonicLFOs",      "contrib.harmonic_lfos.HarmonicLFOs"],
    ["HelloWorld",        "contrib.hello_world.HelloWorld"],
    ["Itty Bitty",        "contrib.itty_bitty.IttyBitty"],
    ["KnobPlayground",    "contrib.knob_playground.KnobPlayground"],
    ["Kompari",           "contrib.kompari.Kompari"],
    ["Logic",             "contrib.logic.Logic"],
    ["Lutra",             "contrib.lutra.Lutra"],
    ["MasterClock",       "contrib.master_clock.MasterClock"],
    ["NoddyHolder",       "contrib.noddy_holder.NoddyHolder"],
    ["Pam's Workout",     "contrib.pams.PamsWorkout2"],
    ["Particle Phys.",    "contrib.particle_physics.ParticlePhysics"],
    ["Pet Rock",          "contrib.pet_rock.PetRock"],
    ["Piconacci",         "contrib.piconacci.Piconacci"],
    ["PolyrhythmSeq",     "contrib.polyrhythmic_sequencer.PolyrhythmSeq"],
    ["PolySquare",        "contrib.poly_square.PolySquare"],
    ["Probapoly",         "contrib.probapoly.Probapoly"],
    ["Quantizer",         "contrib.quantizer.QuantizerScript"],
    ["RadioScanner",      "contrib.radio_scanner.RadioScanner"],
    ["Scope",             "contrib.scope.Scope"],
    ["Seq. Switch",       "contrib.sequential_switch.SequentialSwitch"],
    ["Sigma",             "contrib.sigma.Sigma"],
    ["Slopes",            "contrib.slopes.Slopes"],
    ["Smooth Rnd Volts",  "contrib.smooth_random_voltages.SmoothRandomVoltages"],
    ["StrangeAttractor",  "contrib.strange_attractor.StrangeAttractor"],
    ["Traffic",           "contrib.traffic.Traffic"],
    ["Turing Machine",    "contrib.turing_machine.EuroPiTuringMachine"],
    ["Volts",             "contrib.volts.OffsetVoltages"],

    # System tools, in alphabetical order with a _ prefix
  
    ["_About",            "tools.about.About"],
    ["_BootloaderMode",   "bootloader_mode.BootloaderMode"],
    ["_Calibrate",        "tools.calibrate.Calibrate"],
    ["_Config Editor",    "tools.conf_edit.ConfigurationEditor"],
    ["_Diagnostic",       "tools.diagnostic.Diagnostic"],
    ["_Exp Cfg Editor",   "tools.experimental_conf_edit.ExperimentalConfigurationEditor"],
])
# fmt: on

if __name__ == "__main__":
    BootloaderMenu(EUROPI_SCRIPTS).main()
